package QNetwork;

/*Questa classe contiene l'algoritmo vero e proprio, che, ricevuta una rete
 *in input (sotto forma di oggetto QNet) l'analizza producendo i risultati 
 *desiderati.
 */
class Analyser {
    //mu[i][j] contiene il tasso d'arrivo dal nodo j alla subnetwork composta
    //dal nodo i 
    private double mu[][]; 
    //mu[i][j][r] e' il tasso d'arrivo dal nodo j al nodo i degli utenti della
    //catena r
    private double mur[][][];
    //mdr[i][r] e' il tasso di servizio del nodo i per la catena r
    private double mdr[][];
    //ps[i][r] e' la probabilita' che alla fine di un servizio ad un utente 
    //della catena r, la sottorete T(i) rimanga priva di utenti di quella catena
    private double ps[][];
    //pb[i][j][k] e' la probabilita' che la j-esima sorgente trovi k altre
    //sorgenti bloccate al nodo i
    private double pb[][][];
    private QNet qnetwork;
    public int[][] Ui, Di;
    private double mrstar[][];
    private double xuj[][]; //throughput entrante al nodo i per ogni sorgente j
    private double xdr[][]; //throughput uscente dal nodo i per ogni catena r
    private int M,R; //M e' il numero di nodi e R il numero di catene
    private double epsilon=0.00001;
	
    //il costruttore inizializza alcune variabili e alloca la memoria
    public Analyser(QNet net) {
    	int i;
        qnetwork = net;
        M=qnetwork.getNodes();
        R=qnetwork.getChains();
        Ui = new int[M][0];
        Di = new int[M][0];
        initUi();
        initDi();
        mu = new double[M][];
        mur = new double[M][][];
        mdr = new double[M][R];
        ps = new double[M][R];
        mrstar = new double[M][R];
        xuj = new double[M][];
        xdr = new double[M][R];
        pb = new double[M][][];
        for(i=0;i<M;i++){
            pb[i]= new double[Ui[i].length][Ui[i].length];
            xuj[i] = new double[Ui[i].length];
            mu[i]=new double[Ui[i].length];
            mur[i]= new double[Ui[i].length][R];
        }
			
    }	
	
    //inizializza l'insieme Ui che contiene tutte le possibili sorgenti di ogni
    //nodo
    //0 e' la sorgente esterna.. i nodi quindi partono da 1
    private void initUi(){
        int i,j,r,l,h,n;
        boolean test=false;
        int [] temp;

        for(i=0;i<M;i++) {
            //inserisco la sorgente esterna
            for(n=0;n<R;n++)
                if(qnetwork.getArriveRate(i,n)>0)
                    test = test || true;
            if(test){
                Ui[i] = new int[1];
                Ui[i][0] = 0; 
            }
            test = false;
            for(j=0;j<M;j++) {
                for(r=0;r<R;r++) {
                    //controlla nella matrice di routing di ogni catena r se il
                    //nodo j e' una possibile sorgente del nodo i
                    if(qnetwork.getProb(r,j,i)>0)
                        test = test || true;
                }
                //se j puo' essere sorgente, lo inserisce nell'insieme
                if(test){ 
                    l= (Ui[i].length)+1;
                    temp = new int[l];
                    for(h=0;h<Ui[i].length;h++)
                        temp[h]=Ui[i][h];
                    temp[h]=j+1;
                    Ui[i]=temp;
                }
                test=false;					
            }
        }	
    }
	
    //inizializza l'insieme Di che contiene tutte le possibili destinazioni
    //di ogni nodo
    //0 e' la destinazione esterna.. i nodi quindi partono da 1
    private void initDi(){
        int i,j,r,l,h,n;
        boolean test=false;
        int [] temp;
        for(i=0;i<M;i++) {
            //inserisco la sorgente esterna
            for(n=0;n<R;n++)
                if(qnetwork.getOutProb(n,i)>0)
                    test = test || true;
            if(test){
                Di[i] = new int[1];
                Di[i][0] = 0; 
            }
            test = false;
            for(j=0;j<M;j++) {
                for(r=0;r<R;r++) {
                    //controlla nella matrice di routing di ogni catena r se il
                    //nodo j e' una possibile destinazione del nodo i
                    if(qnetwork.getProb(r,i,j)>0)
                        test = test || true;
                }
                if(test){
                    l= Di[i].length+1;
                    temp = new int[l];
                    for(h=0;h<Di[i].length;h++)
                        temp[h]=Di[i][h];
                    temp[h]=j+1;
                    Di[i]=temp;
                }
                test=false;					
            }
        }
    }
	
    //permette di impostare la precisione da avere nella condizione di uscita
    //di default epsilon=1.0E-5
    public void setEpsilon(double newEpsilon){	
        epsilon=newEpsilon;
    }
		
    //e' il metodo principale ed applica l'algoritmo per il calcolo delle
    //prestazioni
    public Result analyzeNow(){
	
        int i,j,u1,u2,r,k,count=0,unit;
        double sum,diff=0,sum1=0,mdtemp=0;
        double throughputIn=0,throughputOut=0;
        boolean test=false;
        //solo per i test!
        double mdrTot=0,mdrMed=0,rho=0,muTot=0;
        double verifica [][] = new double[M][2];
		
        //Inizializzazione
        for(i=0;i<M;i++){
            sum=0;
            verifica[i][0]=0;
            verifica[i][1]=0;
            if(Ui[i][0]==0){
                test=true;
                for(r=0;r<R;r++){
                    mur[i][0][r] = qnetwork.getArriveRate(i,r);
                    sum+=mur[i][0][r];				
                }
                mu[i][0]=sum;
            }
            if(test)
                j=1;
            else
                j=0;
            for(;j<Ui[i].length;j++){  //MODIFICATO 8.1.05:prima tutto uguale a zero
                if(Ui[i][j]!=0){
                    //mu[i][j]=0;
                    sum=0;
                    for(r=0;r<R;r++){
                        mur[i][j][r]=qnetwork.getServiceRate(Ui[i][j]-1,r)*qnetwork.getProb(r,Ui[i][j]-1,i);
                        sum+=mur[i][j][r];
                    }
                    mu[i][j]=sum;
                } 
            }
            test=false;
        }
        for(i=0;i<M;i++)
            for(r=0;r<R;r++)
                mdr[i][r]=qnetwork.getServiceRate(i,r);
			
                
        for(i=0;i<M;i++)
            for(u1=0;u1<Ui[i].length;u1++)
                for(u2=0;u2<Ui[i].length;u2++)
                    pb[i][u1][u2]=0;				
				
					
        //Fine Inizializzazione
        do{
            if(count>500)
                System.out.println("qui");
            //forward cycle
            for(i=0;i<M;i++){
                //calcolo ps in un modo valido solo quando R=1..sar da correggere...
                for(r=0;r<R;r++)
                    if(i==0)
                        ps[i][r]=(piGreco(i, r,1))/(1-piGreco(i,r,0));
                    else
                        ps[i][r]=(piGrecoBR(i,r,1))/(1-piGrecoBR(i,r,0));
                for(j=0;j<Ui[i].length;j++){
                    mu[i][j]=0;
                    for(r=0;r<R;r++)
                        mu[i][j]+=setMur(i,j,r);
                }
            }
            //backward cycle
            for(i=M-1;i>=0;i--){
                for(r=0;r<R;r++)
                    setMdr(i,r);
                if(Ui[i][0]==0)
                    unit=1;
                else unit=0;
                if(i==7)
                    System.out.println("qui");
                for(j=(0+unit);j<Ui[i].length;j++){
                    if(Ui[i][j]!=0)
                        for(k=0;k<(Ui[i].length-unit);k++){
                            setPb(i,j,k);
                        }				
                }
            }
						
            //calcolo il throughput in entrata e in uscita per ogni nodo.
            //il throughput in uscita viene calcolato per ogni catena.
            for(i=0;i<M;i++){
                for(j=0;j<Ui[i].length;j++)
                    xuj[i][j]=setXuj(i,j);
                for(r=0;r<R;r++)
                    xdr[i][r]=setXdr(i,r);
            }
			
            test=false;
            diff=sum1;
            sum1=0;
            //verifica la condizione d'uscita
            for(i=0;i<M;i++){
			
                throughputIn=0;
                for(j=0;j<Ui[i].length;j++)
                    throughputIn+=xuj[i][j];
                throughputOut=0;
                for(j=0;j<Ui[i].length;j++){
                    for(r=0;r<R;r++){
                        u1=Ui[i][j]-1;
                        if(Ui[i][j]==0)
                            throughputOut+=qnetwork.getArriveRate(i,r);
                        else throughputOut+=(xdr[Ui[i][j]-1][r]*qnetwork.getProb(r,Ui[i][j]-1,i)); //modificato!!!!! 5/12
                    }	}
                System.out.println(count+"iterazioni:troughput finale "+throughputIn+"diff:"+(throughputIn-throughputOut));
                sum1+=throughputIn+throughputOut;
                diff-=(throughputIn+throughputOut);
                //test= test && ((throughputIn-throughputOut)<epsilon);
				
                mdtemp=0;
                for(r=0;r<R;r++)
                    mdtemp+=mdr[i][r];
                verifica[i][1]= Math.abs(1/mdtemp-verifica[i][0]);
                verifica[i][0]=1/mdtemp;				
            }
		
            for(i=0;i<M;i++)
                if(verifica[i][1]<epsilon)
                    test=true;
            count++;
		
            //	if(test)
            //		System.out.println("e' true per "+count+"iterazioni...troughput finale:"+(throughputIn-throughputOut));
            //	System.out.println(diff);
            //}while(count<10 && (Math.abs(diff)>=epsilon || diff==(Double.NaN)));
        }while(count<1000 && (!test || (Math.abs(diff)>=epsilon || diff==(Double.NaN))));
        //}while(count<500);
		
	//	System.out.println("La rete e' stata risolta in "+(count)+" iterazioni");
        for(i=0;i<M;i++){
            mdrTot=0;
            muTot=0;
            throughputIn=0;
            for(j=0;j<Ui[i].length;j++)
                throughputIn+=xuj[i][j];
            throughputOut=0;
            for(j=0;j<Ui[i].length;j++){
                for(r=0;r<R;r++){
                    u1=Ui[i][j]-1;
                    if(Ui[i][j]==0)
                        throughputOut+=qnetwork.getArriveRate(i,r);
                    else throughputOut+=(xdr[Ui[i][j]-1][r]*qnetwork.getProb(r,Ui[i][j]-1,i)); //modificato 5/12 era Ui[i][j],i+1
                }	}
				
            //System.out.println("U:"+ rho +" AvNan:"+(rho/((1-rho)*(1-rho)))+" AvTIQ:"+ ((rho*rho)/(1-rho)));
        }
			
			
        return new Result(mu,mur,mdr,xuj,xdr,qnetwork,count);
		
    }
	
    //calcola il throughput in entrata che node riceve dalla sorgente source
    private double setXuj(int node,int source){
							
        int k,i;
        double temp=0,mdrTot=0;
        for(i=0;i<R;i++)
            mdrTot+=mdr[node][i];
        for(k=0;k<Ui[node].length-1;k++)
            temp += pb[node][source][k] * ((k+1)/mdrTot);	
        return 1/((1/mu[node][source])+temp);
    }
	
    //calcola il throughput in uscita di node per la catena chain		
    private double setXdr(int node, int chain){
        setMrstar(node,chain);					
        if(mrstar[node][chain]==0)
            return 0;
        return 1/((1/mdr[node][chain])+(ps[node][chain]/mrstar[node][chain]));	
    }
	
    //imposta la variabile pb[node][source][k] secondo la formula 9.18
    private void setPb(int node, int source, int k){		
        int j;
        double pi,buj=0,bujpartial=0;
        if(Ui[node].length==1 && Ui[node][source]==0){      //DA VERIFICARE
            pb[node][source][k]=0;
            return;
        }
             
        pi=piGrecoB(node,(k+qnetwork.getCapacity(node)+1)); //aggiunto +1...il server	
        if(k!=0){        
            bujpartial=setBuj(node,source,k,pi);
            for(j=1;j<Ui[node].length;j++)  //MODIFICATO: era strettamente minore!
                buj+=setBuj(node,source,j,pi);
        }else bujpartial=0;	//MODIFICATO: tolto l'else			
        pb[node][source][k]=(pi-bujpartial)/(1-buj);
	//	if(pb[node][source][k]!=0)
	//		System.out.println("node "+node+" source "+source+" k "+k+" pb:"+pb[node][source][k]);	
    }

    //calcola la variabile buj secondo la formula 9.21
    private double setBuj(int node, int source, int k, double piGreco){
		
	double temp;
	temp=(mu[node][source]*omega(node,k-1,source))/omega(node,k,-1);
	return temp*piGreco;					
    }
        
        
    //j,k,i come da notazione Lee Pollock
    public double omega(int j, int k, int i){
        int [] set;
        int n,m,plus=0,U;
        U = Ui[j].length;
        if(i>=0)
            U--;
        if(Ui[j][0]==0 && i!=0)
            U--;        
        if(k==0)
            return 1;
        //faccio una veloce verifica del campo di esistenza
        if(k>U || U==0 || i>Ui[j].length)
            System.out.println("Errore: metodo omega.. U="+U+" node="+j+" k="+k+" i="+i);
        boolean increment = false;
        set = new int[U];
        int [] selected = new int[k];
        double sum=0,prod=1;
        //inizializzazione
        //inizializzo l'insieme degli elementi da combinare

        for(n=0;n<set.length;n++){
            if(n==i || Ui[j][n]==0)
                plus++;
            set[n] = n+plus;
        }
        //prima combinazione
        for(n=0;n<k;n++)
            selected[n] = n;
        do{
            
            for(n=0;n<k;n++)
                prod*=mu[j][set[selected[n]]];
            sum+=prod;
            prod=1;
            
            increment= true;
            for(n=k-1;n>=0;n--){
                if(increment)
                    if(selected[n]<set.length-(k-n)){   
                        selected[n]++;
                        for(m=n+1;m<k;m++)
                            selected[m] = selected[n]+(m-n);
                        increment=false;
                    }
            }
            
        }while(!increment);
        return sum;
    }
    
    //j,k,i come da notazione Lee Pollock
    public double omegaR(int j, int k, int i,int chain){
        int [] set;
        int n,m,plus=0,U;
        U = Ui[j].length;
        if(i>=0)
            U--;
        if(Ui[j][0]==0 && i!=0)
            U--;        
        if(k==0)
            return 1;
        //faccio una veloce verifica del campo di esistenza
        if(k>U || U==0 || i>Ui[j].length)
            System.out.println("Errore: metodo omega.. U="+U+" node="+j+" k="+k+" i="+i);
        boolean increment = false;
        set = new int[U];
        int [] selected = new int[k];
        double sum=0,prod=1;
        //inizializzazione
        //inizializzo l'insieme degli elementi da combinare

        for(n=0;n<set.length;n++){
            if(n==i || Ui[j][n]==0)
                plus++;
            set[n] = n+plus;
        }
        //prima combinazione
        for(n=0;n<k;n++)
            selected[n] = n;
        do{
            
            for(n=0;n<k;n++)
                prod*=mur[j][set[selected[n]]][chain];
            sum+=prod;
            prod=1;
            
            increment= true;
            for(n=k-1;n>=0;n--){
                if(increment)
                    if(selected[n]<set.length-(k-n)){   
                        selected[n]++;
                        for(m=n+1;m<k;m++)
                            selected[m] = selected[n]+(m-n);
                        increment=false;
                    }
            }
            
        }while(!increment);
        return sum;
    }
    
    
    //calcola Omega seconda secondo la formula 9.18
    private double setOmega(int node, int k){
		
    	int n=k;
    	int U;
    	boolean reset;
    	int array[]= new int[n];
    	int i,j,m=0,l;
    	double sum=0,prod=1;
    	if(Ui[node].length<1 || k==0)
            return 1;
    	if(Ui[node][0]==0)
            m=1;
    	U=(Ui[node].length)-m;
    	for(i=0;i<n;i++)
            array[i]=i+1;
       	if(k==0 || U<=0 || n==0)
            return 1;
    	while(array[0]<=(U-(n-1))) {
    		
            for(l=0;l<n;l++) {
                prod*=mu[node][array[l]-1+m];
            }
            sum+=prod;
            prod=1;
       
            reset=false;
 
            for(i=n-1;i>=0;i--){
    			
                if(reset){
                    array[i]++;
                    if(array[i]<=(U-(n-(i+1)))){
                        for(j=1;(i+j)<=(n-1);j++)
                            array[i+j]=array[i]+j;
                        reset=false;
                    }
                }else if(i==n-1){
    				
                    array[i]++;
                    if(array[i]>(U-(n-(i+1))))
                        reset=true;
    			
                }
    			
            }
        
    	}
        //System.out.println(sum);
	return sum;					
    }	
	
    //calcola la probabilita' che in node ci siano n utenti della catena chain
    public double piGreco(int node, int chain, int n){		
	
        double rho,rhon=1,rhon2=1,murTot=0,muTot=0,mdrTot=0,ris;
        int i;
        for(i=0;i<Ui[node].length;i++){
            murTot+=setMrstar(node,chain);  //6.2 modificato
            //murTot+=mur[node][i][chain];
        }
        rho = murTot/mdr[node][chain];
        //  if(rho>1)   // MODIFICA 3.1.05
        //      rho=1;
        if(!(rho>0 || rho<0))
            return 0;
		
        for(i=0;i<n;i++)
            rhon*=rho;
        ris=rhon*(1-rho);
        ///// AGGIUNTO 16.1.05
        if(qnetwork.getCapacity(node)>0){
            for(i=0;i<qnetwork.getCapacity(node)+1;i++)
                rhon2*=rho;
            ris/=1-rhon2;
        }
        ///FINE AGGIUNTA
        return ris;  
    }
        
    //calcola la probabilita' che in node ci siano n utenti della catena chain
    public double piGreco(int node, int n){		
        double sum=0,prod;
        int i,m;
        if(R==1)
            return piGreco(node, 0, n);
        m = n;
        for(i=0;m-i>=0;i++){
            sum+=piGreco(node,0,m-i)*piGreco(node,1,i);
        }
        return sum;
    }
        
    //calcola la probabilita' che in node ci siano n utenti della catena chain
    /*public double piGreco(int node, int n){		
	
    double rho,rhon=1,rhon2=1,murTot=0,muTot=0,mdrTot=0,ris;
    int i,c;
    for(i=0;i<Ui[node].length;i++){
    for(c=0;c<R;c++)
    murTot+=setMrstar(node,c);  //6.2 modificato
    //murTot+=mur[node][i][chain];
    }
    rho = murTot/mdTot(node);
    //  if(rho>1)   // MODIFICA 3.1.05
    //      rho=1;
    if(!(rho>0 || rho<0))
    return 0;
		
    for(i=0;i<n;i++)
    rhon*=rho;
    ris=rhon*(1-rho);
    ///// AGGIUNTO 16.1.05
    if(qnetwork.getCapacity(node)>0){
    for(i=0;i<qnetwork.getCapacity(node)+1;i++)
    rhon2*=rho;
    ris/=1-rhon2;
    }
    ///FINE AGGIUNTA
    return ris;  
    }*/
        
	
        
    public double piGrecoB(int node, int n){
        double sum=0,prod;
        int i,m;
        if(R==1)
            return piGrecoBR(node, 0, n);
        m = n;
        for(i=0;m-i>=0;i++){
            sum+=piGrecoBR(node,0,m-i)*piGrecoBR(node,1,i);
        }
        return sum;
    }
        
	
    /*public double piGrecoB(int node, int n){		
	
    int i,k,j;
    double sum,prod,p0,lambda,mi;
    int num;
    int capacity = qnetwork.getCapacity(node);
    if(qnetwork.getCapacity(node)>=0)  //provo a considerare qua dentro anche la coda infinita
    num = qnetwork.getCapacity(node)+(Ui[node].length-1);
    else{
    num=200;
    capacity = 200;
    }
    //calcolo la prob che il nodo sia vuoto
    sum=0;
    for(i=0;i<num;i++){
    prod=1;
    for(k=0;k<=i;k++){
    if(k<capacity){
    lambda=0;
    for(j=0;j<R;j++)
    lambda=lambda+setMrstar(node,j);
    }else
    //corretto 10.1.05: prima era solo k il primo elemento del prodotto
    lambda=((k-capacity+1)*omega(node,(k-capacity+1),-1)/omega(node,k-capacity,-1));
    //			if((node==5 || node==3) && k>=qnetwork.getCapacity(node))
    //				System.out.println(setOmega(node,k-1)+" e "+k);
    mi=0;
    for(j=0;j<R;j++)
    mi+=mdr[node][j];
    //mi = mdTot(node); //dovrebbe essere pi corretto calcolare md totale cos
    prod*=lambda/mi;
    }
			
    sum+=prod;
    }
    sum++; //NO.. ricorretto! 8.1.05 ho commentato queste due righe e ho scritto quella sotto

    p0=1/sum;
    if(n==0)
    return p0;
    //p0 = 1-sum;
    prod=1;
    for(k=1;k<=n;k++){ 
    if(k<=capacity || (Ui[node].length==1 && Ui[node][0]==0)){
    lambda=0;
    for(j=0;j<R;j++)
    lambda+=mrstar[node][j];
    }else
    lambda=(k*omega(node,k-capacity,-1))/omega(node,k-capacity-1,-1);
    mi=0;
    for(j=0;j<R;j++)
    mi+=mdr[node][j];
    // mi = mdTot(node);
    //if(k>=qnetwork.getCapacity(node))
    //	System.out.println(k+" ecco:"+lambda);
    prod*=lambda/mi;
    }
    //	System.out.println("node:"+node+" n:"+n+" prob:"+(p0*prod));
    //	if((p0*prod)>0)
    //		System.out.println("la prob che al nodo"+node+"ci siano "+n+" utenti e':"+(p0*prod));
    return p0*prod;
    }	*/
	
    //da finire... calcola la probabilit solo per catena r
    public double piGrecoBR(int node, int chain, int n){		
	

        int i,k,j;
        double sum,prod,p0,lambda,mi;
        int num;
        int capacity = qnetwork.getCapacity(node);
        if(qnetwork.getCapacity(node)>=0)  //provo a considerare qua dentro anche la coda infinita
            num = qnetwork.getCapacity(node)+(Ui[node].length-1);
        else{
            num=200;
            capacity = 200;
        }
        //calcolo la prob che il nodo sia vuoto
        sum=0;
        for(i=0;i<num;i++){
            prod=1;
            for(k=0;k<=i;k++){
                if(k<=capacity+1){  //modificato! 13 febbraio
                    lambda=0;
                    lambda=lambda+setMrstar(node,chain);  //MODIFICATO FINO A QUI!!!
                }else
                    //corretto 10.1.05: prima era solo k il primo elemento del prodotto
                    lambda=((k-capacity+1)*omegaR(node,(k-capacity+1),-1,chain)/omegaR(node,k-capacity,-1,chain));
		//			if((node==5 || node==3) && k>=qnetwork.getCapacity(node))
		//				System.out.println(setOmega(node,k-1)+" e "+k);
                mi=0;
                //for(j=0;j<R;j++)
                //	mi+=mdr[node][j];
                //mi = mdTot(node);
                mi = mdr[node][chain];
                prod*=lambda/mi;
            }
			
            sum+=prod;
        }
        sum++; //NO.. ricorretto! 8.1.05 ho commentato queste due righe e ho scritto quella sotto

        p0=1/sum;
        if(n==0)
            return p0;
        //p0 = 1-sum;
        prod=1;
        for(k=1;k<=n;k++){ 
            if(k<=capacity+1 || (Ui[node].length==1 && Ui[node][0]==0)){
                lambda=0;
                lambda+=mrstar[node][chain];
            }else
                lambda=(k*omegaR(node,k-capacity,-1,chain))/omegaR(node,k-capacity-1,-1,chain);
            mi=0;
            //for(j=0;j<R;j++)
            //	mi+=mdr[node][j];
            //mi = mdTot(node);
            mi = mdr[node][chain];
            //if(k>=qnetwork.getCapacity(node))
            //	System.out.println(k+" ecco:"+lambda);
            prod*=lambda/mi;
        }
	//	System.out.println("node:"+node+" n:"+n+" prob:"+(p0*prod));
	//	if((p0*prod)>0)
	//		System.out.println("la prob che al nodo"+node+"ci siano "+n+" utenti e':"+(p0*prod));
        return p0*prod;
    }	
        
        
    //imposto la variabile mur[node][source][chain] secondo la formula 10.3
    private double setMur(int node, int source, int chain){  //DA QUI
	
        int nodeSource=0,k;
        double temp1=0,temp2=0;
        if(source>=Ui[node].length)
            return 0;

        if(Ui[node][source]==0){
            mur[node][source][chain]=qnetwork.getArriveRate(node,chain);
        }else{
            nodeSource = Ui[node][source]-1;
            setMrstar(nodeSource,chain);
            if(qnetwork.getProb(chain,nodeSource,node)==0)
                return 0;
            if(qnetwork.getCapacity(node)!=-1 && Ui[node][source]!=0)
		for(k=0;k<Ui[node].length;k++){ //Modificato 14.1.05 era Ui[node].length-1
                    //   if(mdTot(node)!=0)  
                    //       temp1 += pb[node][source][k] * ((k+1)/mdTot(node));
                    temp1 += pb[node][source][k] * ((k+1)/mdAvg(node)); 
                }
            temp2 = ps[nodeSource][chain]/mrstar[nodeSource][chain];
               
            temp2 += 1/mdr[nodeSource][chain];
            //               if(node==2)
            //   System.out.println("qui");
            temp2 -= qnetwork.getProb(chain,nodeSource,node)*temp1;

            mur[node][source][chain]=qnetwork.getProb(chain,nodeSource,node)*(1/temp2);
        }
        return mur[node][source][chain];
    }
	
    //calcola mrstar[node][chain] secondo la formula 10.3
    private double setMrstar(int node, int chain){		
        mrstar[node][chain]=0;
        for(int i=0;i<Ui[node].length;i++){
            if(mur[node][i][chain]>0 || mur[node][i][chain]<0)
                mrstar[node][chain]+=mur[node][i][chain];
        }
        return mrstar[node][chain];
    }
	
    //calcola mdr[node][chain] secondo la formula 10.4
    private void setMdr(int node, int chain){
		
        int i,m,k,t=0,h;
        double temp1=0,temp2=0;
        for(i=0;i<Di[node].length;i++){
            if(Di[node][i]!=0){
                m = Di[node][i]-1; 	
                //node e' la t-esima sorgente di m
                for(h=0;h<Ui[m].length;h++){				
                    if(Ui[m][h]-1==node)
                        t=h;	
                }
                if(qnetwork.getCapacity(m)!=-1)
                    for(k=0;k<=Ui[m].length-1;k++){ //modifica 9.1.05 era strettamente minore
                        //temp1 += pb[m][t][k] * ((k+1)/mdTot(m));  //Considero il 
                        temp1 += pb[m][t][k] * ((k+1)/mdAvg(m));
                    }
                temp2 += qnetwork.getProb(chain,node,m)*temp1;
                temp1=0; //modifica 13.2.05
            }
        }
        mdr[node][chain]=1/((1/qnetwork.getServiceRate(node,chain))+temp2);	
    }
        
    //calcola mdr[node][chain] secondo la formula 10.4
    /*private void setMdr(int node, int chain){
		
    int i,m,k,t=0,h;
    double temp1=0,temp2=0;
    for(i=0;i<Di[node].length;i++){
    if(Di[node][i]!=0)
    temp1+=qnetwork.getProb(chain,node,Di[node][i]-1)*Eij(node,chain,i);
    }
    mdr[node][chain]=1/temp1;	
    }
       

    private double Eij(int node, int chain, int j){
		
    int m,k,t=0,h;
    double temp1=0,temp2=0;
    if(Di[node][j]!=0){
    m = Di[node][j]-1; 	
    //node e' la t-esima sorgente di m
    for(h=0;h<Ui[m].length;h++){				
    if(Ui[m][h]-1==node)
    t=h;	
    }
    if(qnetwork.getCapacity(m)!=-1)
    for(k=0;k<=Ui[m].length-1;k++){ //modifica 9.1.05 era strettamente minore
    temp1 += pb[m][t][k] * ((k+1)/mdr[node][chain]);
    }
    }
	
    return 1/qnetwork.getServiceRate(node,chain)+temp1;	
    }*/
	
    private double mdAvg(int node){
        double mdTot=0;
        double muTot=0;
        int i;
        for(i=0;i<R;i++)
            muTot += mrstar[node][i];
        for(i=0;i<R;i++)
            //if(qnetwork.getLambda(node)!=0) // ho commentato xk non so a cosa serve!!!!
            mdTot+= mdr[node][i]*(mrstar[node][i]/muTot);
        //mdTot+= mdr[node][i]*(qnetwork.getLambdaR(node,i)/qnetwork.getLambda(node));
        return mdTot;
    }	
        
    private double mdTot(int node){
        double mdTot=0;
        double muTot=0;
        int i;
        for(i=0;i<R;i++)
            muTot += mrstar[node][i];
        for(i=0;i<R;i++)
            mdTot+= mdr[node][i];
        //mdTot+= mdr[node][i]*(qnetwork.getLambdaR(node,i)/qnetwork.getLambda(node));
        return mdTot;
    }	
}
