package QNetwork;
//gli oggetti di questa classe contengono i risultati dell'analisi della rete
//fatta con Analyser

import java.lang.Math;

class Result {
	//mu[i][j] contiene il tasso d'arrivo dal nodo j alla subnetwork composta
	//dal nodo i 
	private double mu[][]; 
	//mu[i][j][r] e' il tasso d'arrivo dal nodo j al nodo i degli utenti della
	//catena r
	private double mur[][][];
	//mdr[i][r] e' il tasso di servizio del nodo i per la catena r
	private double mdr[][];
	private double xuj[][]; //throughput entrante al nodo i per ogni sorgente j
	private double xdr[][]; //throughput uscente dal nodo i per ogni catena r
        private double rho[][]; //calcola il rho per ogni nodo e catena
        private int M=0,R=0;
        private QNet network;
        private int iterazioni;
	
	 public Result(double mu[][], double mur[][][], double mdr[][], double xuj[][], double xdr[][],QNet net,int iter) 
	 {
                int i,r,j;
                double murTot; //variabile temporanea che memorizza il totale input di catena R su un nodo
                this.mu=mu;
	 	this.mur=mur;
	 	this.mdr=mdr;
	 	this.xuj=xuj;
	 	this.xdr=xdr;
                network = net;
                M = mdr.length;
                if(M!=0)
                    R = mdr[0].length;
                rho = new double[M][R];
                for(i=0;i<M;i++)
                    for(r=0;r<R;r++){
                        murTot=0;
                        for(j=0;j<mur[i].length;j++)
                            murTot+=mur[i][j][r];
                        rho[i][r] = murTot/mdr[i][r]; 
                       // if(rho[i][r]>1)
                       //     rho[i][r]=1;
                    }
                iterazioni = iter;
                
	 }
         
         public double getRho(int node, int chain){
             return rho[node][chain];
         }
         
         public int getNIter(){
             return iterazioni;
         }
         
         //probabilita' che al nodo node ci siano n utenti della catena chain
         public double getPiR(int node, int chain, int n){
             //proviamo...
             if(rho[node][chain]==1)
                 rho[node][chain]=0.99;
            if(network.getCapacity(node)==-1) //ipotezzi che rho^infinity tenda  a zero
             return (1-rho[node][chain])*Math.pow(rho[node][chain],n);
            else
             return ((1-rho[node][chain])*Math.pow(rho[node][chain],n))/(1-Math.pow(rho[node][chain],network.getCapacity(node)+1));
         }  
         
         //probabilita' che al nodo node ci siano n utenti della catena chain
         //intanto  compatibile con una sola catena
         public double getPiR(int node, int chain, int n, Analyser analizzatore){
             if(node==0 && network.getCapacity(node)==-1)
                 return analizzatore.piGreco(node,chain,n);
             
             return analizzatore.piGrecoBR(node,chain,n);
         }  
         
          public double getPi(int node, int n, Analyser analizzatore){
             if(node==0 && network.getCapacity(node)==-1)
                 return analizzatore.piGreco(node,n);
             
             return analizzatore.piGrecoB(node,n);
         }  
	 
	 public double [][] getMu(){
			return mu;	    	
	 }
	 
	 public double [][][] getMur(){
			return mur;	    	
	 }
	 
	 public double [][] getMdr(){
			return mdr;	    	
	 }
	 
	 public double [][] getXuj(){
			return xuj;	    	
	 }

    public double getX( int j ) {
        double tput = 0.0;
        int ch; // catena
        for ( ch=0; ch<R; ch++ ) {
            tput += xdr[j][ch];
        }
        return tput;
    }
	 
	 public double [][] getXdr(){
			return xdr; 	
	 }	 
}
