/****************************************************************************
 *
 * qn_server.hh
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef QN_SERVER_HH
#define QN_SERVER_HH

#include "cppsim.hh"
#include <string>
#include <vector>
#include <queue>

class job;

//
// class Server
//
class server: public cppsim::process {
public:
    server( const std::string& name, int serverno );
    virtual ~server( );

    // Typedefs
    typedef enum { IDLE, RUNNING, BLOCKED } status_t;

    // Modifiers
    void add_new_job( job* j ); // add a new job
    void add_waiting_proc( const cppsim::handle_p& p ); // add a new waiting process
    void setStatus( status_t s );

    // Accessors (const)
    status_t getStatus( void ) const { return _status; };

    bool full( void ) const { 
        return ( !( ( _capacity < 0 ) || ( _jobs.size( ) < _capacity ) ) );
    };
    int maxlen( void ) const { return _capacity; };
    int qlen( void ) const { return _jobs.size(); };
    int completed( void ) const { return _completed; }; 

protected:
    // Modifiers
    void remove_first_job( void );

    // Accessors
    // void dump_status( void );

    void inner_body( void );

    const int _serverno; // server ID
    const int _capacity; // Maximum queue length
    std::queue< job* > _jobs;
    std::queue< cppsim::handle_p > _blocked;
    status_t _status;
    // bool _busy;
    std::vector< cppsim::rng* > _servTime;
    std::vector< cppsim::rngDiscEmpirical* > _nextServer;
    // Cached variables
    cppsim::timeweighted* _utilization;
    cppsim::timeweighted* _realutilization;
    cppsim::timeweighted* _queue_length;
    cppsim::timewhisto* _queue_p;
    int _completed; // Number of completed jobs
};

#endif
