/****************************************************************************
 *
 * qn_source.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "qn_source.hh"
#include "qn_server.hh"
#include "qn_job.hh"
#include "qn.hh"

using namespace std;
using namespace cppsim;

//
// Class source
//
source::source( const string& name, int jclass, double jrate, int server_id ) :
    process( name ),
    _jclass( jclass ),
    _r( new rngExp( oform( "source_%d_%d_rng", server_id, jclass ), 1.0/jrate ) ),
    _sid( server_id )
{ 
    // cout << "Source for server " << server_id
    // << " class " << jclass << " rate " << jrate << endl;
}

void source::inner_body( void )
{   
    handle<server> s( servers[ _sid ] );

    while ( 1 ) {
        if ( !s->full( ) ) {
            job* j( new job( _jclass ) );
            s->add_new_job( j );
            if ( server::IDLE == s->getStatus( ) ) {
                message( this, "Server %d available\n", _sid );
                s->activateAfter( current( ) );
            }
        } else {
            message( this, "Server %d full. job dropped\n", _sid );
        }
        hold( _r->value( ) );
    }
}

source::~source( )
{
    delete _r;
}
