/****************************************************************************
 *
 * qn_topology.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include <iostream>
#include <fstream>
#include <string>
#include <cassert>
#include "qn_topology.hh"

using namespace std;

qnet::qnet( const string& fname )
{
    ifstream is( fname.c_str() );
    unsigned int i,j,k;
    assert( is );
    string s;

#ifdef NEW_VERSION
    while ( is >> s ) {
        if ( s == "nclasses" ) {
            is >> _nclasses;
            assert( _nclasses < NCLASSES );
        } else if ( s == "nservers" ) {
            is >> _nservers;
            assert( _nservers < NSERVERS );
        } else if ( s == "topology" ) {
            // get topology
            for ( k=0; k<_nclasses; k++ ) 
                for ( i=0; i<_nservers; i++ ) 
                    for ( j=0; j<_nservers; j++ ) 
                        is >> _topology[k][i][j];
        } else if ( s == "arrivals" ) {
            // get arrivals
            for ( k=0; k<_nclasses; k++ )
                for ( i=0; i<_nservers; i++ )
                    is >> _arrivals[k][i];
        } else if ( s == "services" ) {
            // get service rates
            for ( k=0; k<_nclasses; k++ )
                for ( i=0; i<_nservers; i++ )
                    is >> _service[k][i];
        } else if ( s == "capacities" ) {
            // get capacities
            for ( i=0; i<_nservers; i++ )
                is >> _capacity[i];
        } else {
            cerr << "Unrecognized command " << s << endl;
            exit( -1 );
        }
    }
#else

    // Get nclasses
    is >> _nclasses;
    assert( _nclasses < NCLASSES );

    // Get nservers
    is >> _nservers;
    assert( _nservers < NSERVERS );

    // get topology
    for ( k=0; k<_nclasses; k++ ) 
        for ( i=0; i<_nservers; i++ ) 
            for ( j=0; j<_nservers; j++ ) 
                is >> _topology[k][i][j];

    // get arrivals
    for ( k=0; k<_nclasses; k++ )
        for ( i=0; i<_nservers; i++ )
            is >> _arrivals[k][i];

    // get service rates
    for ( k=0; k<_nclasses; k++ )
        for ( i=0; i<_nservers; i++ )
            is >> _service[k][i];

    // get capacities
    for ( i=0; i<_nservers; i++ )
        is >> _capacity[i];

#endif
}

void qnet::dump( void ) const
{
    unsigned int i,j,k;
    
    cout << "nclasses " << _nclasses << endl
         << "nservers " << _nservers << endl;

    for (k=0; k<_nclasses; k++ ) {
        cout << "Routing for class " << k << endl;
        for (i=0; i<_nservers; i++ ) {
            for ( j=0; j<_nservers; j++ ) {
                cout << _topology[k][i][j] << "\t";
            }
            cout << endl;
        }
        cout << endl;
    }

    for (k=0; k<_nclasses; k++ ) {
        cout << "Arrivals for class " << k << endl;
        for (i=0; i<_nservers; i++ ) 
            cout << _arrivals[k][i] << "\t";
        cout << endl << endl;
    }

    for (k=0; k<_nclasses; k++ ) {
        cout << "Service rates for class " << k << endl;
        for (i=0; i<_nservers; i++ ) 
            cout << _service[k][i] << "\t";
        cout << endl << endl;
    }

    cout << "Capacities " << endl;
    for (i=0; i<_nservers; i++ ) 
        cout << _capacity[i] << "\t";
    cout << endl;
}
