/****************************************************************************
 *
 * qn_topology.hh
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef QN_TOPOLOGY_HH
#define QN_TOPOLOGY_HH

#include <cassert>
#include <string>

#define NCLASSES 5
#define NSERVERS 20

using namespace std;

class qnet {
public:
    qnet( unsigned int nc, unsigned int ns ) :
        _nclasses( nc ),
        _nservers( ns )
    {

    };
    qnet( const string& fname );
    virtual ~qnet( ) { };
    //
    // Modifiers
    //
    int& capacity( unsigned int n ) {
        assert( n < _nservers );
        return _capacity[n];
    };
    double& service( unsigned int c, unsigned int n ) {
        assert( n < _nservers );
        assert( c < _nclasses );
        return _service[c][n];
    };
    double& topology( unsigned int c, unsigned int n1, unsigned int n2 ) {
        assert( n1 < _nservers );
        assert( n2 < _nservers );
        assert( c < _nclasses );
        return _topology[c][n1][n2];
    };
    double& arrivals( unsigned int c, unsigned int n ) {
        assert( n < _nservers );
        assert( c < _nclasses );
        return _arrivals[c][n];
    };
    //
    // Accessors
    //
    unsigned int nclasses ( void ) const {
        return _nclasses;
    };
    unsigned int nservers( void ) const {
        return _nservers;
    };
    void dump( void ) const;
protected:

    unsigned int _nclasses;
    unsigned int _nservers;

    double      _topology[NCLASSES][NSERVERS][NSERVERS];
    double      _arrivals[NCLASSES][NSERVERS];
    double      _service[NCLASSES][NSERVERS];
    int         _capacity[NSERVERS];
};

#endif
