/*****************************************************************************
 *
 * accum.hh
 *
 * Definition of an accumulator class. Accumulators collect the integral
 * of some time-dependent function.
 * This file is part of libcppsim.
 *
 * Copyright (C) 2002, 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef _CPPSIM_ACCUM_HH
#define _CPPSIM_ACCUM_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <utility>
#include <stdexcept>
#include "var.hh"

namespace cppsim {

    class accum : public var< const std::pair<double, double>&, double > {
    public:
        typedef var< const std::pair<double, double> &, double > var_impl;

        accum( const std::string& name );
        virtual ~accum( );
        // Modifiers
        void update( const var_input_t val_time ) throw( std::runtime_error );
        void reset( void );
        // Accessors
        double value( void ) { return _sum; };
        double lastUpdate( void ) const { return _lastUpdate; };
    protected:
        double _sum;           // Sum of the values
        double _lastUpdate;    // Time of last update
        double _lastVal;       // Last value inserted
    };

}

#endif
