/****************************************************************************
 *
 * assertions.hh
 *
 * Improved assertions for C++.
 * This file is part of libcppsim
 *
 * Copyright (C) 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_ASSERTIONS_HH
#define _CPPSIM_ASSERTIONS_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdexcept>
#include "logger.hh"

namespace cppsim {

#ifndef NDEBUG

    // This function should be used instead of the classic assert(...)
    // Inspired from http://www.cs.wustl.edu/~schmidt/PDF/C++-assert4.pdf
    template< class Exception >
    inline void __t_assert( bool cond, const Exception& e,
                            const char* ex, const char* file, int line ) {
        if ( ! cond ) {
            logger::instance( "cppsim.exceptions" )->message( logger::LOG_CRITICAL, "Assertion \"%s\" failed on line %d file \"%s\"\n", ex, line, file );
            throw e;
        }
    }

#define Assert( cond, except ) __t_assert( cond, except, #cond, __FILE__, __LINE__ )
#define Assert1( cond ) __t_assert( cond, runtime_error( "Runtime Error" ), #cond, __FILE__, __LINE__ )

#else   // NDEBUG is defined

#define Assert( cond, except ) (0)
#define Assert1( cond ) (0)

#endif  // defined(NDEBUG)

}

#endif
