/****************************************************************************
 *
 * cdf.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2005 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "cdf.hh"
#include <cmath>

namespace cppsim {

    double cdfUniform01::value( double x )
    {
        if ( x < 0.0 )
            return 0.0;
        else {
            if ( x > 1.0 )
                return 1.0;
            else
                return x;
        }
    }

    double cdfUniform::value( double x )
    {
        if ( x < _a )
            return 0.0;
        else {
            if ( x > _b )
                return 1.0;
            else
                return (x-_a)/(_b-_a);
        }
    }

    double cdfExp::value( double x ) 
    {
        return ( 1.0 - exp( -x/_mean ) );
    }

    double cdfDiscUniform::value( double x )
    {
        if ( x < _a )
            return 0.0;
        else {
            if ( x > _b )
                return 1.0;
            else
                return (double)( x-_a+1.0 ) / ( _b-_a+1.0 );
        }
    }

    double cdfWeib::value( double x ) 
    {
        return 1.0 - exp( -pow(x/_a,_b) );
    }

    double cdfErl::value( double x )
    {
        return 0.0; // FIXME
    }

    double cdfBernoulli::value( double x )
    {
        return 0.0; // FIXME
    }

    double cdfBinomial::value( double x )
    { 
        return 0.0; // FIXME
    }

    double cdfNormal::value( double x )
    {
        return 0.0; // FIXME
    }

    double cdfGamma::value( double x )
    {
        return 0.0; // FIXME
    }

    double cdfChisq::value( double x )
    {
        return 0.0; // FIXME
    }

}
