/*****************************************************************************
 *
 * coroutine.hh
 *
 * This file is part of libccppsim
 *
 * Copyright (C) 2002, 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef CPPSIM_COROUTINE_HH
#define CPPSIM_COROUTINE_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#if USE_MCSC // makecontext/setcontext
extern "C" {
#include <ucontext.h>
}
#elif USE_SJLJ // setjmp/longjmp
extern "C" {
#include <setjmp.h>
}
#include <cstdlib>
#else

#error Undefined coroutine method

#endif                          // End alternative part

namespace cppsim {

    class coroutine
    {

#if USE_MCSC
        friend void cor_trampoline( coroutine * c );
#endif                          // End alternative part

    public:
#if USE_SJLJ
        static char* _stackBottom;
#endif
        virtual ~coroutine ();
    
        // Accessors
        bool terminated( void ) const;
    
        // Modifiers
        void resume( void );
        void call( void );
    
        static size_t cStackSize;
    protected:
        coroutine( );
        void detach( void );
        virtual void main( void ) = 0;              // Coroutine body
    
        static unsigned long _num_ctx_switch;	// Number of context switches
    private:
        inline void enter( void );

#if USE_MCSC
        ucontext_t  *_context;
        bool        _cor_terminated;
#else                                   // setjmp/longjmp, copy
        inline void storeStack( void );
        inline void restoreStack( void );

        jmp_buf     _envir;
        char        *_stackBuffer, *_low, *_high;
        size_t      _bufferSize;
#endif

        coroutine   *_caller;
        coroutine   *_callee;
        static coroutine *_current_coroutine;
    };


#if USE_SJLJ
   
#define INIT_SEQUENCING { char dummy; cppsim::coroutine::_stackBottom=&dummy; }

#else

#define INIT_SEQUENCING

#endif


}

#endif
