/*****************************************************************************
 *
 * counter.cc
 *
 * Definition of a counter class.
 *
 * Copyright (C) 2002 Moreno Marzolla, Paolo Palmerini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <iostream>
#include <cstdio>
#include "counter.hh"

namespace cppsim {

    using namespace std;

    counter::counter( const string& name ) :
        var_impl    ( name, reportable::STATISTIC ),
        _min        ( -1 ),
        _max        ( -1 ),
        _sum        ( 0.0 )
    {

    }

    counter::~counter( )
    {

    }

    void counter::update( double v )
    {
        var_impl::update( v );
        if ( -1 == _min || v < _min )
            _min = v;
        if ( -1 == _max || v > _max )
            _max = v;
        _sum += v;
    }

    void counter::reset( void )
    {
        var_impl::reset( );
        _min = _max = -1;
        _sum = 0.0;
    }

}
