/****************************************************************************
 *
 * logger.hh
 *
 * Logging facility.
 * This file is part of libcppsim.
 *
 * Copyright (C) 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_LOGGER_HH
#define _CPPSIM_LOGGER_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <cstdarg>
#include <iostream>
#include <map>
#include <string>

namespace cppsim {

    class logger;

    class logger {
    public:
        typedef enum { LOG_INFO     =1, 
                       LOG_DEBUG    =2, 
                       LOG_WARNING  =4, 
                       LOG_CRITICAL =8, 
                       LOG_FATAL    =16 } loglevel_t;

        virtual ~logger( ) { };

        // Accessors
        static logger* instance( const std::string& name );
        bool enabled( int l ) const { return ( l & _l ); };

        // Modifiers
        void enable( int l ) 	{ _l |= l; };
        void disable( int l ) 	{ _l ^= l; };
	void set( int l ) 	{ _l = l; };
        void setOutputFile( std::ostream* os );
        void message( loglevel_t l, const char* fmt, ... );
        void vmessage( loglevel_t l, const char* fmt, va_list ap );

    protected:
        logger( const std::string& name );

	const std::string	_name;
        std::ostream*    _os;    // Output stream
        int              _l;     // Level of verbosity
        static std::map< std::string, logger* > _instances;
    };

}

#endif
