/****************************************************************************
 *
 * repSimulation.hh
 *
 * This file is part of libcppsim.
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_REPSIMULATION_HH
#define _CPPSIM_REPSIMULATION_HH

#include "simulation.hh"
#include "process.hh"   // (r)
#include <string>

namespace cppsim {
    
    class repSimulation : public simulation {
    public:
        repSimulation( double simDuration,
                       int maxReplications,
                       double relWidth );
        virtual ~repSimulation( ) { };
        void run( void );
    protected:
        virtual void initialize( void ) = 0;
        virtual void finalize( void ) = 0;
        void reset( void );
        bool terminated( void ); 

        //
        // This process is used to stop the simulation after a certain 
        // user-defined duration, and return control to the repSimulation
        // class for checking the termination condition.
        //
        class guard : public process {
        public:
            guard( double duration ) : 
                process( "repSimulation::guard" ),
                _duration( duration )
            { };
            virtual ~guard( ) { };
        protected:
            void inner_body( void );
            double _duration;
        };

        double _simDuration;    // Duration of a single run
        int _maxReplications;   // Max number of independent runs
        double _relWidth;       // Accuracy (relative width)
        int _nrep;              // Current number of replications
    };
    
}

#endif
