/****************************************************************************
 *
 * reportable.hh
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2005 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_REPORTABLE_HH
#define _CPPSIM_REPORTABLE_HH

#include <string>
#include <vector>

namespace cppsim {

    using std::string;
    using std::vector;
    
    class reportable {
    public:
        typedef enum {
            RNG = 0,
            STATISTIC,
            TRREMOVAL,
            UNKNOWN
        } repClass_t;

        virtual ~reportable( ) { };
        // Accessors
        int getClass( void ) const { return _repClass; };
        bool hidden( void ) const { return _repHidden; };
        virtual string title( void ) const;
        virtual const vector< string > headers( void ); // FIXME: should be pure virtual
        virtual const vector< string > entries( void ); // FIXME; should be pure virtual
        // Modifiers
        void setClass( int c ) { _repClass = c; };
        void setHidden( bool h ) { _repHidden = h; };
    protected:
        reportable( int c = UNKNOWN ) : 
            _repClass( c ),
            _repHidden( false )
        { };

        int _repClass;   // The class of this reportable object
        bool _repHidden; // true iff this object should not be reported
    };

}

#endif
