/****************************************************************************
 *
 * simpleReport.cc
 *
 * This file is part of libcppsim.
 *
 * Copyright (C) 2005 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "simpleReport.hh"
#include <iostream>
#include <vector>
#include <string>
#include "oform.hh" // (r)
#include "reportable.hh"

namespace cppsim {

    using namespace std;
    
    void simpleReport::print( void )
    {
        cout << endl
             << "*** Report " << name() << " ***" << endl
             << endl << endl;
        list< reportable* >::const_iterator it;
        for ( it = _registered.begin(); it != _registered.end(); it++ ) {
            reportable* r = *it;
            if ( ! r->hidden() ) {
                cout << " " << r->title() << endl;
                printEntry( r );
            }
        }
        
    };

    void simpleReport::printEntry( reportable* r )
    {
        unsigned int i;
        const vector< string > h( r->headers() );
        const vector< string > v( r->entries() );
        for ( i = 0; i<h.size(); i++ ) {
            cout << oform( "  %20s %20s", h[i].c_str(), v[i].c_str() )
                 << endl;
        }
    }

};
