/****************************************************************************
 *
 * simulation.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "simulation.hh"
#include "sqs.hh"
#include "report.hh"
#include "assertions.hh"        // (r)

namespace cppsim {

    using namespace std;

    list<simulation*> simulation::_sim = list<simulation*>( );

    simulation::simulation( void ) :
        _sqs( defaultSqsBuilder() ),
        _report( 0 ),
        _rngs( ),
        _stats( ),
        _reps( )
    {
        _sim.push_front( this );
    }

    simulation::~simulation( )
    {
        Assert1( ! _sim.empty() );
        simulation* s = _sim.front( );
        Assert1( s == this );
        _sim.pop_front( );
        delete _sqs;
        delete _report;
    }

    sqs* simulation::getSqs( void ) const
    {
        return _sqs;
    }

    report* simulation::getReport( void ) const
    {
        return _report;
    }

    void simulation::setReport( report* r )
    {
        delete _report;
        _report = r;
    }

    void simulation::registerRng( rng* r )
    {
        _rngs.push_back( r );
    }

    void simulation::registerStatistic( statistic* s )
    {
        _stats.push_back( s );
    }

    void simulation::registerReportable( reportable* r )
    {
        assert( 0 != _report );
        _report->registerObj( r );
        _reps.push_back( r );
    }

    unsigned long simulation::events( void ) const
    {
        return _sqs->processed( );
    }

    simulation* simulation::instance( void )
    {
        Assert1( ! _sim.empty() );
        return _sim.front( );
    }

    list<rng*> simulation::getRngs( void ) const
    {
        return _rngs;
    }

    list<statistic*> simulation::getStatistics( void ) const
    {
        return _stats;
    }

    list<reportable*> simulation::getReportables( void ) const
    {
        return _reps;
    }

}
