/****************************************************************************
 *
 * simulation.hh
 *
 * Abstract simulation class.
 * This file is part of libcppsim.
 *
 * Copyright (C) 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_SIMULATION_HH
#define _CPPSIM_SIMULATION_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <list>

namespace cppsim {

    class sqs;
    class rng;
    class statistic;
    class reportable;
    class report;

    class simulation  {
    public:

        virtual ~simulation( );

        virtual void run( void ) = 0;

        // Modifiers
        void setReport( report* r );
        void registerRng( rng* r );
        void registerStatistic( statistic* s );
        void registerReportable( reportable* r );
        static simulation* instance( void );

        // Accessors
        report* getReport( void ) const;
        std::list<rng*> getRngs( void ) const;
        std::list<statistic*> getStatistics( void ) const; 
        std::list<reportable*> getReportables( void ) const;
        unsigned long events( void ) const;
        sqs* getSqs( void ) const;

    protected:
        simulation( );

        sqs* _sqs;
        report* _report;
        std::list<rng*> _rngs;
        std::list<statistic*> _stats;
        std::list<reportable*> _reps;
        static std::list<simulation*> _sim;
    };

}

#endif
