/****************************************************************************
 *
 * statistic.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "statistic.hh"
#include "assertions.hh"        // (r)
#include "oform.hh"             // (r)

namespace cppsim {

    using std::ostream;
    using std::domain_error;

    ostream& operator<<( ostream& os, const confInt& c )
    {
        os << c.val() << " ("
           << c.lBound() << ","
           << c.uBound() << ") "
           << c.confl()*100 << "%";
        return os;
    }

    confInt::confInt( double lBound, double uBound, double confl ) 
        throw( domain_error ):
	_lBound( lBound ),
	_uBound( uBound ),
	_confl( confl )
    {
	Assert( lBound <= uBound, 
                domain_error (
                              oform( "confInt(lBound=%f,uBound=%f,confl=%f) "
                                     "must have lBound < uBound", 
                                     lBound, uBound, confl ) 
                              )
                );
	Assert( 0.0 < confl && confl < 1.0,
                domain_error (
                              oform( "confInt(lBound=%f,uBound=%f,confl=%f) "
                                     "must have 0<confl<1", 
                                     lBound, uBound, confl ) 
                              )
                );
    };
    
    statistic::statistic( const std::string& name, double confl ) 
        throw( domain_error ) :
	var_impl        ( name, reportable::STATISTIC ),
	_confl          ( confl )
    { 
	Assert( 0.0 < confl && confl < 1.0,
                domain_error( 
                             oform( "statistic(\"%s\", confl=%f) "
                                    "must have 0<confl<1", name.c_str(), confl )
                             )
                );
    };

}
