/****************************************************************************
 *
 * timeweighted.hh
 *
 * This file is part of libcppsim.
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_TIMEWEIGHTED_HH
#define _CPPSIM_TIMEWEIGHTED_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>
#include "statistic.hh"
#include "repmean.hh"
#include "trremoval.hh" // (r)

namespace cppsim {

    // class welch;

    class timeweighted : public var< const std::pair<double, double>&, confInt > {
    public:
        // Typedefs
        typedef var< const std::pair<double, double>&, confInt > var_impl;

        timeweighted( const std::string& name,
                      double confl,
                      double bwidth,
                      trremoval* t = 0
                      );
        virtual ~timeweighted( );

        // Modifiers
        void update( const std::pair< double, double> & v );
        void reset( void );
        confInt value( void );
    protected:
        const double _confl;
        const double _bwidth; // Width of the batch
        double _startTime; // Time the last batch began
        double _lastUpdated; // Time the last value was added
        double _lastVal; // The last value added
        double _sum; // The weighted sum of the current batch
        repmean _r;
        // welch* _w;
    };

}

#endif
