/****************************************************************************
 *
 * timewhisto.hh
 *
 * This file is part of libcppsim.
 *
 * Copyright (C) 2005 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef _CPPSIM_TIMEWISTO_HH
#define _CPPSIM_TIMEWHISTO_HH

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>
#include <vector>
#include "statistic.hh"
#include "trremoval.hh" // (r)

namespace cppsim {

    class timeweighted;

    class timewhisto : public var< const std::pair<int, double>&, confInt > {
    public:
        // Typedefs
        typedef var< const std::pair<int, double>&, confInt > var_impl;

        timewhisto( const std::string& name,
                    int nmax,
                    double confl,
                    double bwidth
                    );
        virtual ~timewhisto( );

        // Accessors
        const std::vector< std::string > headers( void );
        const std::vector< std::string > entries( void );

        // Modifiers
        void update( const std::pair< int, double > & v );
        void reset( void );
        confInt value( void );
    protected:
        int _nmax;
        double _confl;
        std::vector< timeweighted* > _bins;
    };

}

#endif
