/****************************************************************************
 *
 * trremoval.cc
 *
 * This file is part of libcppsim.
 *
 * Copyright (C) 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "trremoval.hh"
#include "oform.hh" // (r)

namespace cppsim {

    using namespace std;

    trremovalBuilderFrac __b( 0.2 );
    trremovalBuilder_t* trremovalDefaultBuilder = &__b;

    //
    // trremoval
    //
    const vector< string > trremoval::headers( void ) 
    {
        vector< string > result(3);
        result[0] = "Name";
        result[1] = "Total Events";
        result[2] = "Discarded Events";
        return result;
    }

    //
    // trremoval_const
    //
    void trremoval_const::update( double v )
    {
        var_impl::update( v );
        if ( numUpdates() > _l )
            _ob.push_back( v );
    }

    void trremoval_const::reset( void ) 
    {
        var_impl::reset( );
        _ob.clear();
    }

    const vector< string > trremoval_const::entries( void )
    {
        vector< string > result(3);
        result[0] = name();
        result[1] = oform("%u", numUpdates() );
        result[2] = oform("%u", _l );
        return result;
    }

    //
    // trremoval_frac
    //
    void trremoval_frac::update( double v ) 
    {
        var_impl::update( v );
        _ob.push_back( v );
    }

    void trremoval_frac::reset( void ) 
    {
        var_impl::reset( );
        _ob.clear();
    }

    vector<double> trremoval_frac::value( void ) throw( runtime_error )
    {
        if ( _ob.size() > 1 ) {
            return vector<double>( _ob.begin()+(unsigned int)(_ob.size()*_f), _ob.end() );
        } else {
            return vector<double>( );
        }
    }

    const vector< string > trremoval_frac::entries( void )
    {
        vector< string > result(3);
        result[0] = name();
        result[1] = oform("%u", numUpdates() );
        result[2] = oform("%u", (int)( _f*_ob.size() ) );
        return result;
    }

}
