/*****************************************************************************
 *
 * var.hh
 *
 * Definition of an abstract class for variables.
 * This file is part of libcppsim.
 *
 * Copyright (C) 2003, 2004, 2006 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef CPPSIM_VAR_HH
#define CPPSIM_VAR_HH

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>
#include "reportable.hh"

namespace cppsim {

    //////////////////////////////////////////////////////////////////////////
    //
    // Class var. 
    //
    // A Variable is an object which can be updated with objects of
    // type T_in, and which can return objects of type T_out. Usually,
    // T_in and T_out will both be of type double, or something
    // similar. For statistics, T_in could be of type double, and
    // T_out of type confInt (Confidence Interval)
    //
    //////////////////////////////////////////////////////////////////////////

    template< class T_in, class T_out >
    class var : public reportable {
    public:
        // Typedefs
        typedef T_in var_input_t;
        typedef T_out var_output_t;

        virtual ~var( ) { };

        // Modifiers
        virtual void update( T_in v ) { m_numUpdates++; };
        virtual void reset( void ) { m_numUpdates = 0; m_numResets++; };
        virtual T_out value( void ) = 0;

        // Accessors
        const std::string& name( void ) const { return m_name; };
        unsigned int numUpdates( void ) const { return m_numUpdates; };
        unsigned int numResets( void ) const { return m_numResets; };

    protected:

        var( const std::string& name, int repClass ) : 
            reportable  ( repClass ),
            m_name       ( name ),
            m_numUpdates ( 0 ),
            m_numResets  ( 0 )
        { };

        const std::string m_name;
        unsigned int m_numUpdates;
        unsigned int m_numResets;
    };

}

#endif
