/****************************************************************************
 *
 * quantile_test.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2003, 2004 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include <cmath>
#include <iostream>
#include <iomanip>
#include "cppsim.hh"

using namespace std;
using namespace cppsim;

double compute_quantile( rng* f, double q )
{
    double a = 0.0;
    double b = 10.0;
    cdf* tmpc = f->makeCdf();
    while( (b-a) > 1e-4 ) {
        double fa = tmpc->value( a );
        double fab = tmpc->value( (b+a)/2.0 );
        double fb = tmpc->value( b );
        // assert( fb > q );
        // assert( fa < q );
        if ( fab > q ) 
            b = (b+a)/2.0;
        else
            a = (b+a)/2.0;
    };
    delete tmpc;
    return (b+a)/2.0;
};

int main( void )
{
    int i,j;
    vector< rng* >      rngs;
    vector< quantile* > quas;

    rngs.push_back( new rngUniform01( "Ran01" ) );
    rngs.push_back( new rngExp( "Exp", 2.0 ) );
    rngs.push_back( new rngWeib( "Weib", 1.0, 0.5 ) );

    quas.push_back( new quantile( "Quantile_u01", 0.9, 0.90 ) );
    quas.push_back( new quantile( "Quantile_ex", 0.9, 0.90 ) );
    quas.push_back( new quantile( "Quantile_we", 0.9, 0.90 ) );

    for ( i=0; i<1e6; i++ ) {
        for ( j=0; j<rngs.size(); j++ ) {
            quas[j]->update( rngs[j]->value() );
        }
        if ( (i == 100) || (i == 500) || (i == 1000) || (i == 10000) || (i == 999999) ) {
            cout << "After " << i << " insertions:" << endl
                 << endl << endl
                 << '\t' << "Name"
                 << '\t' << "lBound"
                 << '\t' << "uBound"
                 << '\t' << "exact"
                 << '\t' << "Valid?"
                 << endl;
            for ( j=0; j<rngs.size(); j++ ) {
                confInt ci = quas[j]->value();
                double qq = compute_quantile( rngs[j], 0.9 );

                cout << '\t' << rngs[j]->name()
                     << '\t'
                     << setw(6) 
                     << setprecision(4 ) << ci.lBound() 
                     << '\t' << ci.uBound()
                     << '\t'
                     << qq
                     << '\t'
                     << (ci.contains( qq ) ? 'Y' : 'N') 
                     << endl;
            }
            cout << endl;
        }
    }

    return 0;
}
