/****************************************************************************
 *
 * testTimeWeighted.cc
 *
 * This file is part of libcppsim
 *
 * Copyright (C) 2005 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include <utility>
#include "cppsim.hh"
#include <iostream>
#include <vector>
#include <algorithm>
#include <functional>

using namespace cppsim;
using namespace std;

class foo {
public:
    foo( int bwidth ) :
        _bwidth( bwidth ),
        _startTime( 0.0 ),
        _lastUpdated( 0.0 ),
        _lastVal( 0.0 ),
        _sum( 0.0 ) { };
    virtual ~foo( ) { };
    void update( const pair< double, double>& v );
    void reset( void );
    double value( void ) { return _sum; };
protected:
    int _bwidth;
    double _startTime;
    double _lastUpdated;
    double _lastVal;
    double _sum;
};

void foo::update( const pair< double, double>& v )
{
    double t = v.second;
    double val = v.first;
    while ( t - _startTime > _bwidth ) {
        // Start a new batch
        _sum += ( _startTime + _bwidth - _lastUpdated ) * _lastVal;
        cout << "Batch starting at " << _startTime << " value=" << _sum << endl;
        _startTime += _bwidth;
        _lastUpdated = _startTime;
        _sum = 0;
    }
    _sum += (t - _lastUpdated) * _lastVal;
    _lastVal = val;
    _lastUpdated = t;
}

void foo::reset( void )
{
    _sum = _lastVal = _lastUpdated = _startTime = 0;
}

timeweighted* tl;
vector<timeweighted*> tp(4);

void update( int val, double t )
{
    tl->update( make_pair( val, t ) );
    for ( int i=0; i<tp.size(); i++ ) {
        if ( val == i )
            tp[i]->update( make_pair( 1, t ) );
        else
            tp[i]->update( make_pair( 0, t ) );
    }
}

void summary( void )
{
    confInt cl( tl->value() );
    cout << "L " << cl << endl;
    for ( int i=0; i<tp.size(); i++ ) {
        confInt c( tp[i]->value() );
        cout << "P(" << i << ") " << c << endl;
    }
}

int main( void )
{
    tl = new timeweighted( "", 0.9, 100 );
    tp[0] = new timeweighted( "", 0.9, 100 );
    tp[1] = new timeweighted( "", 0.9, 100 );
    tp[2] = new timeweighted( "", 0.9, 100 );
    tp[3] = new timeweighted( "", 0.9, 100 );
    rngExp x( "interarrival", 5.0 );
    rngDiscUniform y( "users", 0, 3 );

    int i,j;

    for ( i=0; i<10; i++ ) {
        if ( i>0 ) {
            tl->reset();
            vector< timeweighted* >::iterator it;
            for ( it=tp.begin(); it!=tp.end(); it++ )
                (*it)->reset();
        }
        double t=0;
        for ( j=0; j<1000; j++ ) {
            t+=x.value();
            update( (int)y.value(), t );
        }
    }
    summary( );
}
